/*  Hippo Games - 2001 (C) - http://www.prog.cz/hippo/
 *    __________________________________________________
 *   /\     __  __    _____   ______   ______   ______  \
 *   \ \   /\ \|\ \  /\_  _\ /\  __ \ /\  __ \ /\  __ \  \
 *    \ \  \ \ \_\ \ \//\ \/ \ \ \_\ \\ \ \_\ \\ \ \ \ \  \
 *     \ \  \ \  __ \  \ \ \  \ \  __/ \ \  __/ \ \ \ \ \  \
 *      \ \  \ \ \/\ \  \_\ \_ \ \ \/   \ \ \/   \ \ \_\ \  \
 *       \ \  \ \_\ \_\ /\____\ \ \_\    \ \_\    \ \_____\  \
 *        \ \  \/_/\/_/ \/____/  \/_/     \/_/     \/_____/   \
 *         \ \_________________________________________________\
 *          \/_________________________________________________/
 *                           
 * 
 *  Souboru : HD3DX.cpp
 *  
 *  Projekt : HD3DX
 *  
 *  Autor   : Bernard Lidicky [berny@prog.cz]
 *
 *  Datum   : 15.8.2001
 *
 */                 

#include "stdafx.h"
#include "HD3DX.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#pragma comment(lib,"d3d8.lib")
#pragma comment(lib,"d3dx8.lib")



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HD3DX::HD3DX()
{
	for (int i = 0; i < MAX_HD3DX_TEXTURES; i++) {
		m_text[i].d3dt = NULL;
		m_text[i].module = NULL;
		m_text[i].usage = 0;
		m_text[i].name.Empty();
		m_text[i].type = TEXTURE_NONE;
	}
	m_pD3D = NULL;          
	m_dev = NULL;
	m_valid = false;
}

///////////////////////////////////////////////////////////////////////////////////////

HD3DX::~HD3DX()
{

}


///////////////////////////////////////////////////////////////////////////////////////

bool HD3DX::Create(int w, int h, int bpp, bool windowed)
{
	HRESULT hr;
	D3DDISPLAYMODE display_mode;

   //Create Direct3D8, this is the first thing you have to do in any D3D8 program
   //Always pass D3D_SDK_VERSION to the function.
   m_pD3D = Direct3DCreate8( D3D_SDK_VERSION );
   if(!m_pD3D ){
 		AfxMessageBox("Direct3DCreate8");
		exit(1);
  }

   //Get the current(desktop) display mode.  This is really only needed if
   //we're running in a window.
   hr=m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT,&display_mode);
   if(FAILED(hr)){
		AfxMessageBox("Chyba pri GetAdapterDisplayMode");
		exit(1);
   }

   //Clear out our D3DPRESENT_PARAMETERS structure.  Even though we're going
   //to set virtually all of it members, it's good practice to zero it out first.
   ZeroMemory(&m_d3dppPresentParam,sizeof(m_d3dppPresentParam));

   //Whether we're full-screen or windowed these are the same.
   m_d3dppPresentParam.SwapEffect     = D3DSWAPEFFECT_DISCARD; // Throw away previous frames, we don't need them
   m_d3dppPresentParam.hDeviceWindow  = AfxGetMainWnd()->m_hWnd;  //This is our main (and only) window
   m_d3dppPresentParam.BackBufferCount= 1;  //We only need a single back buffer


   // BackBufferWidth/Height have to be set for full-screen apps, these values are
   //used (along with BackBufferFormat) to determine the display mode.
   //They aren't needed in windowed mode since the size of the window will be used.
   // BackBufferFormat is the pixel format we want.  In windowed mode we use the same
   //format as the desktop (which we found by using GetAdapterDisplayMode() above).
   //In full-screen we need to find a pixel format we like, see find_16bit_mode()
   //below for more details.
   if(windowed == false) {
      m_d3dppPresentParam.Windowed          = FALSE;
      m_d3dppPresentParam.BackBufferWidth   = w;
      m_d3dppPresentParam.BackBufferHeight  = h;
      m_d3dppPresentParam.BackBufferFormat  = D3DFMT_A8R8G8B8;
/*		if(!SUCCEEDED(m_pD3D->CheckDeviceType(D3DADAPTER_DEFAULT, //Test the primary display device, this is
                                                 //necessary because systems can have add-on cards
                                                 //or multi-monitor setups
                             D3DDEVTYPE_HAL,  //This states that we want support for this mode
                                              //in hardware rather than emulated in software
                             D3DFMT_A8R8G8B8 ,   //The is the primary (viewable) buffer format
                             D3DFMT_A8R8G8B8 ,   //This is the back (drawable) buffer format
                             FALSE)))   //Is this windowed mode?  Nopp
		{
			AfxMessageBox("Vas hardware nepodporuje D3DFMT_A8R8G8B8");
			exit(1);
		}*/
   } else {
      m_d3dppPresentParam.Windowed          = TRUE;
      m_d3dppPresentParam.BackBufferFormat  = display_mode.Format;
		CRect r1, r2;
		AfxGetMainWnd( )->GetWindowRect(r1);
		AfxGetMainWnd( )->GetClientRect(r2);
		int dw = w - r2.Width() + r1.Width(),
		 	 dh = h - r2.Height() + r1.Height();
		AfxGetMainWnd( )->SetWindowPos(NULL, 0, 0, dw, dh, SWP_NOMOVE | SWP_NOZORDER);
   }

   //After filling in our D3DPRESENT_PARAMETERS structure, we're ready to create our device.
   //Most of the options in how the device is created are set in the D3DPRESENT_PARAMETERS
   //structure.
   hr=m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, //The default adapter, on a multimonitor system
                                              //there can be more than one.
                           //Use hardware acceleration rather than the software renderer
                          D3DDEVTYPE_HAL,
                          //Our Window
                          AfxGetMainWnd()->m_hWnd,
                          //Process vertices in software. This is slower than in hardware,
                          //But will work on all graphics cards.
                          D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                          //Our D3DPRESENT_PARAMETERS structure, so it knows what we want to build
                          &m_d3dppPresentParam,
                          //This will be set to point to the new device
                          &m_dev);
   if(FAILED(hr)){
		AfxMessageBox("Chyba pri tvorbe device");
		exit(1);
   }

	m_bpp = bpp;
	m_windowed = windowed;
	m_w = w;
	m_h = h;

	m_valid = true;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////

D3DFORMAT HD3DX::GetFormat(int bpp)
{
	if (bpp == 32)
		if(SUCCEEDED(m_pD3D->CheckDeviceType(D3DADAPTER_DEFAULT, //Test the primary display device, this is
                                                 //necessary because systems can have add-on cards
                                                 //or multi-monitor setups
                             D3DDEVTYPE_HAL,  //This states that we want support for this mode
                                              //in hardware rather than emulated in software
                             D3DFMT_A8R8G8B8 ,   //The is the primary (viewable) buffer format
                             D3DFMT_A8R8G8B8 ,   //This is the back (drawable) buffer format
                             FALSE)))   //Is this windowed mode?  Nope
		return D3DFMT_A8R8G8B8;
		else {
			AfxMessageBox("Vas hardware nepodporuje D3DFMT_A8R8G8B8");
			exit(1);
		}
	if (bpp == 16)
		if(SUCCEEDED(m_pD3D->CheckDeviceType(D3DADAPTER_DEFAULT, //Test the primary display device, this is
                                                 //necessary because systems can have add-on cards
                                                 //or multi-monitor setups
                             D3DDEVTYPE_HAL,  //This states that we want support for this mode
                                              //in hardware rather than emulated in software
                             D3DFMT_A4R4G4B4 ,   //The is the primary (viewable) buffer format
                             D3DFMT_A4R4G4B4 ,   //This is the back (drawable) buffer format
                             FALSE)))   //Is this windowed mode?  Nope
		return D3DFMT_A4R4G4B4;
		else {
			AfxMessageBox("Vas hardware nepodporuje D3DFMT_A4R4G4B4");
			exit(1);
		}


	return D3DFMT_A8R8G8B8;
}

///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::Destroy()
{
	DestroyTextures();

	if (m_dev != NULL) {
		m_dev->Release();
		m_dev = NULL;
	}

	if (m_pD3D != NULL) {
		m_pD3D->Release();
		m_pD3D = NULL;
	}

}

///////////////////////////////////////////////////////////////////////////////////////

int HD3DX::CreateTextureFromFile(LPCSTR file, DWORD usage, int ID)
{
	if (ID == -1) ID = FindFreeText();

	if (ID == 0) return 0;
	if (!SUCCEEDED(D3DXCreateTextureFromFileEx(m_dev, file, D3DX_DEFAULT, D3DX_DEFAULT,
		0, usage, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, D3DX_FILTER_NONE , D3DX_DEFAULT,
		D3DCOLOR_ARGB(255,255,0,255), &m_text[ID].info, NULL, &m_text[ID].d3dt)))
		return 0;

	m_text[ID].name = file;
	m_text[ID].module = NULL;
	m_text[ID].usage = usage;

	m_text[ID].virtID = 0;
	m_text[ID].type = TEXTURE_REAL;
	m_text[ID].rect.SetRect(0, 0, m_text[ID].info.Width, m_text[ID].info.Height);

	return ID;
}

///////////////////////////////////////////////////////////////////////////////////////

int HD3DX::CreateTextureFromRes(HMODULE modul, LPCSTR resname, DWORD usage, int ID)
{
	CString s;
	s.Format("%s", resname);
	s.MakeUpper();
	if (s.Find(".BMP") != -1) {
		s.Delete(s.GetLength()-4, 4);
	}


	
	if (ID == -1) ID = FindFreeText();
	if (ID == 0) return 0;
	if (!SUCCEEDED(D3DXCreateTextureFromResourceEx(m_dev, modul, s, D3DX_DEFAULT, D3DX_DEFAULT,
		0, usage, D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, D3DX_FILTER_NONE, D3DX_DEFAULT,
		D3DCOLOR_ARGB(255,255,0,255), &m_text[ID].info, NULL, &m_text[ID].d3dt)))
		return 0;

	m_text[ID].name = s;
	m_text[ID].module = modul;
	m_text[ID].usage = usage;
	m_text[ID].type = TEXTURE_REAL;
	m_text[ID].virtID = 0;
	m_text[ID].rect.SetRect(0, 0, m_text[ID].info.Width, m_text[ID].info.Height);

	return ID;
}

///////////////////////////////////////////////////////////////////////////////////////

int HD3DX::CreateTexture(int w, int h, int ID)
{
	if (ID == -1) ID = FindFreeText();
	if (ID == 0) return 0;
	if (!SUCCEEDED(D3DXCreateTexture(m_dev, w, h, 0, D3DUSAGE_RENDERTARGET, D3DFMT_UNKNOWN,
		  D3DPOOL_DEFAULT, &m_text[ID].d3dt)))
		return 0;
	m_text[ID].info.Width = w;
	m_text[ID].info.Height = h;

	m_text[ID].name.Empty();
	m_text[ID].module = NULL;
	m_text[ID].usage = D3DUSAGE_RENDERTARGET;
	m_text[ID].type = TEXTURE_REAL;
	m_text[ID].virtID = 0;
	m_text[ID].rect.SetRect(0, 0, w, h);

	return ID;
}

///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::DestroyTexture(int ID)
{
	if (ID <= 0) return;
	if (m_text[ID].d3dt != NULL) {
		m_text[ID].d3dt->Release();
		m_text[ID].d3dt = NULL;
	}
	m_text[ID].module = NULL;
	m_text[ID].name.Empty();
	m_text[ID].usage = 0;
	m_text[ID].type = TEXTURE_NONE;
}

///////////////////////////////////////////////////////////////////////////////////////

int HD3DX::FindFreeText()
{
	for (int i = 1; i < MAX_HD3DX_TEXTURES; i++) 
		if (m_text[i].type == TEXTURE_NONE) break;
	if (i == MAX_HD3DX_TEXTURES) return 0;
	return i;
}

///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::ErrorMessage(CString mess)
{
	Destroy();
	AfxMessageBox(mess);
	exit(2);
}

///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::BeginScene()
{

	if (m_dev == NULL) return;

	D3DXCreateSprite(m_dev, &m_sprite);

   m_dev->Clear( 0L, NULL, D3DCLEAR_TARGET,
                      0x00000000, 1.0f, 0L );



	if (!SUCCEEDED(m_dev->BeginScene())) {
		ErrorMessage("Begin Scene Failed");
	}

	m_sprite->Begin();
}

///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::EndScene()
{
	m_sprite->End();
	m_dev->EndScene();
	m_sprite->Release();
   m_dev->Present( 0, 0, 0, 0 );

}

void HD3DX::Draw(int sprite, float x, float y, float angle, float sx, float sy, int alpha, int shx, int shy)
{
	Draw(sprite, (int)x, (int)y, angle, sx, sy, alpha, shx, shy);
}

///////////////////////////////////////////////////////////////////////////////////////
/*
  sprite  -  bitmapa
  x       -  kam se ma nakreslit - x
  x       -  kam se ma nakreslit - y
  angle   -  otoceni (v radianech)
  sx      -  zvetseni po x  
  sy      -  zvetseni po y
  alpha   -  polopruhlednost
  shx     -  pri virualnich texturach - kdyz je vice animacnich okenek zasebou, tak ktere po x
  shy     -  animacni okenko po y
*/
void HD3DX::Draw(int sprite, int x, int y, float angle, float sx, float sy, int alpha, int shx, int shy)
{
	D3DXVECTOR2 scaling;
	D3DXVECTOR2 transl;
	D3DXVECTOR2 RotCenter;

	if (m_text[sprite].type == TEXTURE_NONE) {
		return;
	}

	CRect rect;
	int   fsprite = sprite;

	if (m_text[sprite].type == TEXTURE_VIRTUAL)
		fsprite = m_text[sprite].virtID;

	if (m_text[fsprite].d3dt == NULL) return;


	rect.left   = m_text[sprite].rect.left + (m_text[sprite].rect.Width()+1) * shx;
	rect.top    = m_text[sprite].rect.top  + (m_text[sprite].rect.Height()+1) * shy;
	rect.right  = rect.left + m_text[sprite].rect.Width();
	rect.bottom = rect.top  + m_text[sprite].rect.Height();


	scaling.x = sx; scaling.y = sy;

	RotCenter.x = (float)(rect.Width()*sx) / 2;
	RotCenter.y = (float)(rect.Height()*sy) / 2;

	transl.x = (float)x - RotCenter.x; 
	transl.y = (float)y - RotCenter.y;
	m_sprite->Draw(m_text[fsprite].d3dt, &rect, &scaling, &RotCenter, angle, &transl, D3DCOLOR_ARGB(alpha,255,255,255));
}

///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::Resize(int sx, int sy, bool windowed)
{
	if (!m_valid) return;

	if (m_w == sx && m_h == sy && windowed == m_windowed) return;

	int i;
	for (i = 1; i < MAX_HD3DX_TEXTURES; i++) 
		if (m_text[i].d3dt != NULL) {
			m_text[i].d3dt->Release();
			m_text[i].d3dt = NULL;
		}

	if (m_dev != NULL) {
		m_dev->Release();
		m_dev = NULL;
	}
	if (m_pD3D != NULL) {
		m_pD3D->Release();
		m_pD3D = NULL;
	}

	Create(sx, sy, m_bpp, windowed); 

	for (i = 1; i < MAX_HD3DX_TEXTURES; i++)
		ReloadTexture(i);

}

///////////////////////////////////////////////////////////////////////////////////////

bool HD3DX::Restore()
{
	if (!NeedRestore()) return false;



	m_valid = false;

//	Destroy();


	int i;

	TRACE("retextured\n");


	for (i = 1; i < MAX_HD3DX_TEXTURES; i++) {
		if (m_text[i].d3dt != NULL) {
			m_text[i].d3dt->Release();
			m_text[i].d3dt = NULL;
		}
	}

	TRACE("Reset \n");


//	m_dev->Reset(&m_d3dppPresentParam);


	TRACE("Release\n");

	if (m_dev != NULL) {
		m_dev->Release();
		m_dev = NULL;
	}

	if (m_pD3D != NULL) {
		m_pD3D->Release();
		m_pD3D = NULL;
	}

	TRACE("create\n");


	Create(m_w, m_h, m_bpp, m_windowed); 

	for (i = 1; i < MAX_HD3DX_TEXTURES; i++)
		ReloadTexture(i);

	TRACE("completed :=_ \n");

	m_valid = true;

	return true;
}


///////////////////////////////////////////////////////////////////////////////////////

void HD3DX::ReloadTexture(int ID)
{
	if (m_text[ID].type != TEXTURE_REAL) return;

	if (m_text[ID].d3dt != NULL) {
		m_text[ID].d3dt->Release();
		m_text[ID].d3dt = NULL;
	}

	if (m_text[ID].module != NULL)
		CreateTextureFromRes(m_text[ID].module, m_text[ID].name, m_text[ID].usage, ID);
	else
		if (!m_text[ID].name.IsEmpty())
			CreateTextureFromFile(m_text[ID].name, m_text[ID].usage, ID);
		else
			if (m_text[ID].usage == D3DUSAGE_RENDERTARGET) 
				CreateTexture(m_text[ID].info.Width, m_text[ID].info.Height, ID);
}

///////////////////////////////////////////////////////////////////////////////////////

int HD3DX::CreateVirualTexture(int FromID, int left, int top, int w, int h)
{
	int ID = FindFreeText();
	if (ID == 0) return 0;

	m_text[ID].d3dt = NULL;
	m_text[ID].info.Width = w;
	m_text[ID].info.Height = h;

	m_text[ID].name.Empty();
	m_text[ID].module = NULL;
	m_text[ID].usage = 0;
	m_text[ID].type = TEXTURE_VIRTUAL;
	m_text[ID].virtID = FromID;

	SetRect(&m_text[ID].rect, left, top, left+w, top+h);

	return ID;
}

bool HD3DX::NeedRestore()
{
	if (!m_valid || m_windowed) return false;

	if (m_dev->TestCooperativeLevel() == D3D_OK) return false;

	return true;
}

void HD3DX::DestroyTextures()
{
	for (int i = 1; i < MAX_HD3DX_TEXTURES; i++) 
		DestroyTexture(i);
}
